var _path = require("path");
var _fs = require("fs");
var _Object = require("dr-js/library/common/mutable/Object");
var _File = require("dr-js/library/node/file/File");
var _Directory = require("dr-js/library/node/file/Directory");
const loadPackage = (pathInput, path, collect) => {
  const packageSource = (0, _path.relative)(pathInput, path);
  const {dependencies, devDependencies, peerDependencies, optionalDependencies} = JSON.parse((0, _fs.readFileSync)(path, "utf8"));
  dependencies && collect(dependencies, packageSource);
  devDependencies && collect(devDependencies, packageSource);
  peerDependencies && collect(peerDependencies, packageSource);
  optionalDependencies && collect(optionalDependencies, packageSource);
};
exports.collectDependency = (async pathInput => {
  const {collect, getResult} = (() => {
    let packageInfoMap = {};
    let dependencyMap = {};
    return {
      collect: (dependencyObject, source) => Object.entries(dependencyObject).forEach(([name, version]) => {
        if (packageInfoMap[name]) return console.warn(`[collect] dropped duplicate package: ${name} at ${source} with version: ${version}, checking: ${packageInfoMap[name].version}`);
        packageInfoMap[name] = {
          name,
          version,
          source
        };
        dependencyMap[name] = version;
      }),
      getResult: () => {
        (0, _Object.objectSortKey)(dependencyMap);
        const result = {
          packageInfoMap,
          dependencyMap
        };
        packageInfoMap = {};
        dependencyMap = {};
        return result;
      }
    };
  })();
  const isDirectory = (await (0, _File.getPathStat)(pathInput)).isDirectory();
  isDirectory ? (await (0, _Directory.getFileList)(pathInput)).filter(path => path.endsWith("package.json")).forEach(path => loadPackage(pathInput, path, collect)) : loadPackage(pathInput, pathInput, collect);
  const {packageInfoMap, dependencyMap} = getResult();
  return {
    isDirectory,
    packageInfoMap,
    dependencyMap
  };
});