var _format = require("dr-js/library/common/format");
var _function = require("dr-js/library/common/function");
var _SemVer = require("dr-js/library/common/module/SemVer");
var _Run = require("dr-js/library/node/system/Run");
const REGEXP_ANSI_ESCAPE_CODE = /\033\[[0-9;]*[a-zA-Z]/g;
const REGEXP_NPM_OUTDATED_OUTPUT = /(\S+)\s+\S+\s+(\S+)\s+(\S+)/;
const sortTableRow = ([nameA, , , sourceA], [nameB, , , sourceB]) => sourceA !== sourceB ? sourceA.localeCompare(sourceB) : nameA.localeCompare(nameB);
const formatPadTable = table => (0, _format.padTable)({
  table,
  cellPad: " | ",
  padFuncList: [ "R", "L", "L", "L" ]
});
exports.logCheckOutdatedResult = (async (packageInfoMap, pathPackage) => (async (packageInfoMap, npmOutdatedOutputString) => {
  const sameTable = [];
  const outdatedTable = [];
  npmOutdatedOutputString.split("\n").forEach(outputLine => {
    const [, name, versionWanted, versionLatest] = REGEXP_NPM_OUTDATED_OUTPUT.exec(outputLine.replace(REGEXP_ANSI_ESCAPE_CODE, "")) || [];
    if (!packageInfoMap[name]) return;
    const {version, source} = packageInfoMap[name];
    const versionTarget = (0, _SemVer.compareSemVer)(versionWanted, versionLatest) <= 0 ? versionLatest : versionWanted;
    const rowList = [ name, version, versionTarget, source ];
    version.endsWith(versionTarget) ? sameTable.push(rowList) : outdatedTable.push(rowList);
  });
  const total = sameTable.length + outdatedTable.length;
  sameTable.sort(sortTableRow);
  sameTable.length && console.log(`SAME[${sameTable.length}/${total}]:\n${formatPadTable(sameTable)}`);
  outdatedTable.sort(sortTableRow);
  outdatedTable.length && console.error(`OUTDATED[${outdatedTable.length}/${total}]:\n${formatPadTable(outdatedTable)}`);
  return outdatedTable.length;
})(packageInfoMap, await (async pathPackage => {
  const {promise: runPromise, subProcess, stdoutBufferPromise} = (0, _Run.runQuiet)({
    command: "npm",
    argList: [ "--no-update-notifier", "outdated" ],
    option: {
      cwd: pathPackage
    }
  });
  const {promise, resolve, reject} = (0, _function.createInsideOutPromise)();
  runPromise.then(resolve, resolve);
  setTimeout(reject, 42e3);
  const {code, signal} = await promise.catch(async () => {
    console.warn("[checkNpmOutdated] timeout");
    "win32" === process.platform ? (0, _Run.runSync)({
      command: "taskkill",
      argList: [ "-F", "-T", "-PID", subProcess.pid ],
      option: {
        stdio: "ignore"
      }
    }) : subProcess.kill();
    throw new Error("[checkNpmOutdated] timeout");
  });
  return (await stdoutBufferPromise).toString();
})(pathPackage)));