var _path = require("path");
var _fs = require("fs");
var _child_process = require("child_process");
var _format = require("dr-js/library/common/format");
var _Object = require("dr-js/library/common/mutable/Object");
var _File = require("dr-js/library/node/file/File");
var _Modify = require("dr-js/library/node/file/Modify");
var _Run = require("dr-js/library/node/system/Run");
const loadPackage = (packagePath, packageInfo = {
  packageJSON: {},
  exportFilePairList: [],
  installFilePairList: []
}, loadedSet = new Set()) => {
  const packageFile = packagePath.endsWith(".json") ? packagePath : (0, _path.join)(packagePath, "package.json");
  packagePath.endsWith(".json") && (packagePath = (0, _path.dirname)(packagePath));
  if (loadedSet.has(packageFile)) return packageInfo;
  loadedSet.add(packageFile);
  const {IMPORT: importList, EXPORT: exportList, INSTALL: installList, ...mergePackageJSON} = require(packageFile);
  const {packageJSON, exportFilePairList, installFilePairList} = packageInfo;
  importList && importList.forEach(importPackagePath => loadPackage((0, _path.resolve)(packagePath, importPackagePath), packageInfo, loadedSet));
  console.log(`[loadPackage] load: ${packageFile}`);
  installList && installList.forEach(filePath => installFilePairList.push(parseResourcePath(filePath, packagePath)));
  exportList && exportList.forEach(filePath => exportFilePairList.push(parseResourcePath(filePath, packagePath)));
  mergePackageJSON && (0, _Object.objectMergeDeep)(packageJSON, mergePackageJSON);
  return packageInfo;
};
const parseResourcePath = (resourcePath, packagePath) => "object" == typeof resourcePath ? [ (0, _path.resolve)(packagePath, resourcePath.from), resourcePath.to ] : [ (0, _path.resolve)(packagePath, resourcePath), resourcePath ];
const PACKAGE_KEY_SORT_REQUIRED = [ "dependencies", "devDependencies", "peerDependencies", "optionalDependencies", "bundledDependencies" ];
const PACKAGE_KEY_ORDER = [ "private", "name", "version", "description", "author", "contributors", "license", "keywords", "repository", "homepage", "bugs", "os", "cpu", "engines", "engineStrict", "preferGlobal", "main", "bin", "man", "files", "directories", "scripts", "config", "publishConfig", ...PACKAGE_KEY_SORT_REQUIRED, "sideEffects" ];
exports.doPack = (async ({pathInput, pathOutput, outputName, outputVersion, outputDescription, isPublish, isPublishDev}) => {
  const pathOutputInstall = (0, _path.resolve)(pathOutput, "install");
  const {packageJSON, exportFilePairList, installFilePairList} = loadPackage(pathInput);
  outputName && (packageJSON.name = outputName);
  outputVersion && (packageJSON.version = outputVersion);
  outputDescription && (packageJSON.description = outputDescription);
  await _Modify.modify.delete(pathOutput).catch(() => {});
  await (0, _File.createDirectory)(pathOutput);
  await (0, _File.createDirectory)(pathOutputInstall);
  await (async (packageJSON, path) => {
    PACKAGE_KEY_SORT_REQUIRED.forEach(key => {
      packageJSON[key] && (0, _Object.objectSortKey)(packageJSON[key]);
    });
    const jsonFileStringList = Object.keys(packageJSON).sort((a, b) => PACKAGE_KEY_ORDER.indexOf(a) - PACKAGE_KEY_ORDER.indexOf(b)).map(key => (0, _format.stringIndentLine)(`${JSON.stringify(key)}: ${JSON.stringify(packageJSON[key], null, 2)}`));
    const packageBuffer = Buffer.from(`{\n${jsonFileStringList.join(",\n")}\n}\n`);
    (0, _fs.writeFileSync)(path, packageBuffer);
    console.log(`[writePackageJSON] ${path} [${(0, _format.binary)(packageBuffer.length)}B]`);
  })(packageJSON, (0, _path.join)(pathOutput, "package.json"));
  (0, _fs.writeFileSync)((0, _path.join)(pathOutput, "README.md"), [ `# ${packageJSON.name}\n`, "[![i:npm]][l:npm]", "[![i:size]][l:size]", "", `${packageJSON.description}`, "", `[i:npm]: https://img.shields.io/npm/v/${packageJSON.name}.svg?colorB=blue`, `[l:npm]: https://www.npmjs.com/package/${packageJSON.name}`, `[i:size]: https://packagephobia.now.sh/badge?p=${packageJSON.name}`, `[l:size]: https://packagephobia.now.sh/result?p=${packageJSON.name}` ].join("\n"));
  for (const [source, targetRelative] of exportFilePairList) await _Modify.modify.copy(source, (0, _path.join)(pathOutput, targetRelative));
  for (const [source, targetRelative] of installFilePairList) await _Modify.modify.copy(source, (0, _path.join)(pathOutputInstall, targetRelative));
  (0, _child_process.execSync)("npm --no-update-notifier pack", {
    cwd: pathOutput,
    stdio: "inherit",
    shell: !0
  });
  const outputFileName = `${packageJSON.name}-${packageJSON.version}.tgz`;
  const outputFilePath = (0, _path.join)(pathOutput, outputFileName);
  console.log(`done pack: ${outputFileName} [${(0, _format.binary)((0, _fs.statSync)(outputFilePath).size)}B]`);
  (isPublish || isPublishDev) && (0, _Run.runSync)({
    command: "npm",
    argList: [ "publish", outputFilePath, "--tag", isPublishDev ? "dev" : "latest" ]
  });
});