const toExportName = name => `${name.charAt(0).toUpperCase()}${name.slice(1)}`;
const isFirstUpperCase = name => /[A-Z]/.test(name.charAt(0));
exports.generateIndexScript = (({sourceRouteMap}) => {
  const indexScriptMap = {};
  Object.values(sourceRouteMap).forEach(({routeList, directoryList, fileList}) => {
    const textList = [];
    const importList = [];
    directoryList.forEach(name => {
      const exportName = toExportName(name);
      textList.push(`import * as ${exportName} from './${name}'`);
      importList.push(exportName);
    });
    fileList.map(({name, exportList}) => {
      if (directoryList.length || isFirstUpperCase(name)) {
        const exportName = toExportName(name);
        textList.push(`import * as ${exportName} from './${name}'`);
        importList.push(exportName);
      } else textList.push(`export { ${exportList.join(", ")} } from './${name}'`);
    });
    importList.length && textList.push(`export { ${importList.join(", ")} }`);
    indexScriptMap[[ ...routeList, "index.js" ].join("/")] = textList.join("\n");
  });
  return indexScriptMap;
});
exports.HOIST_LIST_KEY = "~hoist";
exports.EXPORT_LIST_KEY = "~export";
exports.EXPORT_HOIST_LIST_KEY = "~export-hoist";
exports.generateExportInfo = (({sourceRouteMap}) => {
  const exportInfoMap = {};
  const getExportInfo = (...routeList) => {
    const key = routeList.join("/");
    exportInfoMap[key] || (exportInfoMap[key] = {});
    return exportInfoMap[key];
  };
  Object.values(sourceRouteMap).forEach(({routeList, directoryList, fileList}) => {
    const exportInfo = getExportInfo(...routeList);
    directoryList.forEach(name => {
      exportInfo[toExportName(name)] = getExportInfo(...routeList, name);
    });
    fileList.map(({name, exportList}) => {
      if (directoryList.length || isFirstUpperCase(name)) exportInfo[toExportName(name)] = {
        "~export": exportList
      }; else {
        exportInfo[name] = {
          "~hoist": exportList
        };
        exportInfo["~export-hoist"] = [ ...exportInfo["~export-hoist"] || [], ...exportList ];
      }
      getExportInfo(...routeList, name)["~export"] = exportList;
    });
  });
  return exportInfoMap;
});