var _path = require("path");
var _fs = require("fs");
var _parser = require("@babel/parser");
var _compare = require("dr-js/library/common/compare");
var _Object = require("dr-js/library/common/mutable/Object");
var _File = require("dr-js/library/node/file/File");
var _Directory = require("dr-js/library/node/file/Directory");
const createExportParser = ({parserPluginList, logger}) => {
  let sourceRouteMap = {};
  const getRoute = routeList => {
    const key = routeList.join("/");
    sourceRouteMap[key] || (sourceRouteMap[key] = {
      routeList,
      directoryList: [],
      fileList: []
    });
    return sourceRouteMap[key];
  };
  return {
    parseExport: async path => {
      const fileStat = await (0, _File.getPathStat)(path);
      const routeList = path.split(_path.sep);
      const name = routeList.pop();
      if (fileStat.isDirectory()) {
        logger.devLog(`[directory] ${path}`);
        getRoute(routeList).directoryList.push(name);
      } else if (fileStat.isFile() && name.endsWith(".js")) {
        const exportList = ((fileString, sourceFilename, parserPluginList) => {
          const exportNodeList = (0, _parser.parse)(fileString, {
            sourceFilename,
            sourceType: "module",
            plugins: parserPluginList || [ "objectRestSpread", "classProperties", "exportDefaultFrom", "exportNamespaceFrom", "jsx" ]
          }).program.body.filter(({type}) => "ExportNamedDeclaration" === type);
          return [].concat(...exportNodeList.map(({specifiers, declaration}) => declaration ? declaration.declarations ? declaration.declarations.map(({id: {name}}) => name) : [ declaration.id.name ] : specifiers.map(({exported: {name}}) => name)));
        })((0, _fs.readFileSync)(path, {
          encoding: "utf8"
        }), path, parserPluginList);
        logger.devLog(`[file] ${path}`);
        if (!exportList.length) return;
        getRoute(routeList).fileList.push({
          name: name.slice(0, -3),
          exportList
        });
        logger.devLog(`  export [${exportList.length}]: ${exportList.join(", ")}`);
      } else logger.devLog(`[skipped] ${path} (${(0, _File.getPathTypeFromStat)(fileStat)})`);
    },
    getSourceRouteMap: () => {
      const result = (sourceRouteMap => {
        Object.values(sourceRouteMap).forEach(({routeList, directoryList, fileList}) => {
          directoryList.sort(_compare.compareString);
          fileList.sort(({name: a}, {name: b}) => (0, _compare.compareString)(a, b));
          fileList.forEach(({exportList}) => exportList.sort(_compare.compareString));
        });
        (0, _Object.objectSortKey)(sourceRouteMap);
        return sourceRouteMap;
      })(sourceRouteMap);
      sourceRouteMap = {};
      return result;
    }
  };
};
exports.createExportParser = createExportParser;
exports.collectSourceRouteMap = (async ({pathRootList = [], pathInfoFilter = info => !0, logger}) => {
  const {parseExport, getSourceRouteMap} = createExportParser({
    logger
  });
  const parseWalkExport = info => pathInfoFilter(info) && parseExport(info.path);
  for (const pathRoot of pathRootList) await (0, _Directory.walkDirectoryInfoTree)(await (0, _Directory.getDirectoryInfoTree)(pathRoot), parseWalkExport);
  return getSourceRouteMap();
});