var _path = require("path");
var _function = require("dr-js/library/node/file/function");
var _generateInfo = require("./generateInfo");
const getMarkdownHeaderLink = text => `[${text}](#${text.trim().toLowerCase().replace(/[^\w\- ]+/g, "").replace(/\s/g, "-").replace(/-+$/, "")})`;
exports.getMarkdownHeaderLink = getMarkdownHeaderLink;
const REGEXP_MARKDOWN_HEADER = /^#{1,6}(.+?)#*$/gm;
exports.autoAppendMarkdownHeaderLink = ((...markdownStringList) => {
  const headerTextList = (string => {
    const headerTextList = [];
    let result;
    for (;result = REGEXP_MARKDOWN_HEADER.exec(string); ) {
      const headerText = result[1].trim();
      headerText && headerTextList.push(headerText);
    }
    return headerTextList;
  })(markdownStringList.join("\n"));
  return headerTextList.length ? [ ...headerTextList.map(text => `* ${getMarkdownHeaderLink(text)}`), "", ...markdownStringList ] : [];
});
const escapeMarkdownLink = name => name.replace(/_/g, "\\_");
exports.escapeMarkdownLink = escapeMarkdownLink;
const renderMarkdownFileLink = path => `📄 [${escapeMarkdownLink(path)}](${path})`;
exports.renderMarkdownFileLink = renderMarkdownFileLink;
exports.renderMarkdownDirectoryLink = (path => `📁 [${escapeMarkdownLink(path).replace(/\/*$/, "/")}](${path})`);
exports.renderMarkdownExportPath = (({exportInfoMap, rootPath}) => Object.entries(exportInfoMap).reduce((textList, [path, value]) => {
  value[_generateInfo.EXPORT_LIST_KEY] && textList.push(`+ ${renderMarkdownFileLink(`${(0, _function.toPosixPath)((0, _path.relative)(rootPath, path))}.js`)}`, `  - ${value[_generateInfo.EXPORT_LIST_KEY].map(text => `\`${text}\``).join(", ")}`);
  return textList;
}, []));
const renderMarkdownExportTree = ({exportInfo, routeList}) => Object.entries(exportInfo).reduce((textList, [key, value]) => {
  if (key === _generateInfo.HOIST_LIST_KEY) ; else if (key === _generateInfo.EXPORT_LIST_KEY || key === _generateInfo.EXPORT_HOIST_LIST_KEY) textList.push(`- ${value.map(text => `\`${text}\``).join(", ")}`); else {
    const childTextList = renderMarkdownExportTree({
      exportInfo: value,
      routeList: [ ...routeList, key ]
    });
    childTextList.length && textList.push(`- **${key}**`, ...childTextList.map(text => `  ${text}`));
  }
  return textList;
}, []);
exports.renderMarkdownExportTree = renderMarkdownExportTree;