var _assert = require("assert");
var _child_process = require("child_process");
var _fs = require("fs");
var _format = require("dr-js/library/common/format");
var _File = require("dr-js/library/node/file/File");
var _Directory = require("dr-js/library/node/file/Directory");
var _Run = require("dr-js/library/node/system/Run");
var _Modify = require("dr-js/library/node/file/Modify");
var _main = require("./main");
exports.initOutput = (async ({fromRoot, fromOutput, deleteKeyList = [ "private", "scripts", "devDependencies" ], copyPathList = [ "LICENSE", "README.md" ], logger: {padLog, log}}) => {
  padLog("reset output");
  await _Modify.modify.delete(fromOutput()).catch(() => {});
  await (0, _File.createDirectory)(fromOutput());
  padLog("init output package.json");
  const packageJSON = require(fromRoot("package.json"));
  for (const deleteKey of deleteKeyList) {
    delete packageJSON[deleteKey];
    log(`dropped key: ${deleteKey}`);
  }
  (0, _fs.writeFileSync)(fromOutput("package.json"), JSON.stringify(packageJSON));
  padLog("init output file");
  for (const copyPath of copyPathList) if ("README.md" === copyPath) {
    (0, _fs.writeFileSync)(fromOutput(copyPath), (0, _fs.readFileSync)(fromRoot(copyPath)).toString().split("[//]: # (NON_PACKAGE_CONTENT)")[0].trim());
    log(`copied: ${copyPath} (with NON_PACKAGE_CONTENT trimmed)`);
  } else {
    await _Modify.modify.copy(fromRoot(copyPath), fromOutput(copyPath));
    log(`copied: ${copyPath}`);
  }
  return packageJSON;
});
exports.packOutput = (async ({fromRoot, fromOutput, logger: {padLog, log}}) => {
  padLog("run pack output");
  (0, _child_process.execSync)("npm --no-update-notifier pack", {
    cwd: fromOutput(),
    stdio: _main.__VERBOSE__ ? "inherit" : [ "ignore", "ignore" ],
    shell: !0
  });
  log("move to root path");
  const packageJSON = require(fromOutput("package.json"));
  const packName = `${packageJSON.name.replace(/^@/, "").replace("/", "-")}-${packageJSON.version}.tgz`;
  await _Modify.modify.move(fromOutput(packName), fromRoot(packName));
  padLog(`pack size: ${(0, _format.binary)((0, _fs.statSync)(fromRoot(packName)).size)}B`);
  return fromRoot(packName);
});
exports.verifyOutputBinVersion = (async ({fromOutput, packageJSON, matchStringList = [ packageJSON.name, packageJSON.version ], logger: {padLog, log}}) => {
  padLog("verify output bin working");
  const outputBinTest = (0, _child_process.execSync)("node bin --version", {
    cwd: fromOutput(),
    stdio: "pipe",
    shell: !0
  }).toString();
  log(`bin test output: ${outputBinTest}`);
  for (const testString of matchStringList) (0, _assert.ok)(outputBinTest.includes(testString), `should output contain: ${testString}`);
});
exports.verifyNoGitignore = (async ({path, logger: {padLog}}) => {
  padLog("verify no gitignore file left");
  const badFileList = (await (0, _Directory.getFileList)(path)).filter(path => path.includes("gitignore"));
  badFileList.length && console.error(`found gitignore file:\n - ${badFileList.join("\n - ")}`);
  (0, _assert.ok)(!badFileList.length, `${badFileList.length} gitignore file found`);
});
const getPublishFlag = flagList => {
  const isDev = flagList.includes("publish-dev");
  return {
    isPublish: isDev || flagList.includes("publish"),
    isDev
  };
};
exports.getPublishFlag = getPublishFlag;
const checkPublishVersion = ({isDev, version}) => isDev ? REGEXP_PUBLISH_VERSION_DEV.test(version) : REGEXP_PUBLISH_VERSION.test(version);
exports.checkPublishVersion = checkPublishVersion;
const REGEXP_PUBLISH_VERSION = /^\d+\.\d+\.\d+$/;
const REGEXP_PUBLISH_VERSION_DEV = /^\d+\.\d+\.\d+-dev\.\d+$/;
exports.publishOutput = (async ({flagList, packageJSON, pathPackagePack, extraArgs = [], logger}) => {
  const {isPublish, isDev} = getPublishFlag(flagList);
  if (!isPublish) return logger.padLog("skipped publish output, no flag found");
  if (!pathPackagePack || !pathPackagePack.endsWith(".tgz")) throw new Error(`[publishOutput] invalid pathPackagePack: ${pathPackagePack}`);
  if (!checkPublishVersion({
    isDev,
    version: packageJSON.version
  })) throw new Error(`[publishOutput] invalid version: ${packageJSON.version}, isDev: ${isDev}`);
  logger.padLog(`${isDev ? "publish-dev" : "publish"}: ${packageJSON.version}`);
  (0, _Run.runSync)({
    command: "npm",
    argList: [ "--no-update-notifier", "publish", pathPackagePack, "--tag", isDev ? "dev" : "latest", ...extraArgs ]
  });
});