var _path = require("path");
var _fs = require("fs");
var _terser = (obj = require("terser")) && obj.__esModule ? obj : {
  default: obj
};
var obj;
var _time = require("dr-js/library/common/time");
var _format = require("dr-js/library/common/format");
var _main = require("./main");
exports.getTerserOption = (({isDevelopment = !1, isModule = !1} = {}) => {
  return {
    ecma: 8,
    toplevel: !0,
    parse: {
      ecma: 8
    },
    compress: {
      ecma: 8,
      toplevel: !0,
      join_vars: !1,
      sequences: !1,
      global_defs: {
        "process.env.NODE_ENV": isDevelopment ? "development" : "production",
        __DEV__: Boolean(isDevelopment)
      }
    },
    mangle: !isModule && {
      toplevel: !0
    },
    output: isModule ? {
      ecma: 8,
      beautify: !0,
      indent_level: 2,
      width: 240
    } : {
      ecma: 8,
      beautify: !1,
      semicolons: !1
    },
    sourceMap: !1
  };
});
const minifyWithTerser = ({filePath, option, logger}) => {
  const timeStart = (0, _time.clock)();
  const scriptSource = (0, _fs.readFileSync)(filePath, {
    encoding: "utf8"
  });
  const {error, code: scriptOutput} = _terser.default.minify(scriptSource, option);
  if (error) {
    logger.padLog(`[minifyWithTerser] failed to minify file: ${filePath}`);
    throw error;
  }
  (0, _fs.writeFileSync)(filePath, scriptOutput);
  const timeEnd = (0, _time.clock)();
  return {
    sizeSource: Buffer.byteLength(scriptSource),
    sizeOutput: Buffer.byteLength(scriptOutput),
    timeStart,
    timeEnd
  };
};
exports.minifyWithTerser = minifyWithTerser;
exports.minifyFileListWithTerser = (async ({fileList, option, rootPath = "", logger}) => {
  logger.padLog(`minify ${fileList.length} file with terser`);
  const table = [];
  let totalTimeStart = (0, _time.clock)();
  let totalSizeSource = 0;
  let totalSizeDelta = 0;
  for (const filePath of fileList) {
    const {sizeSource, sizeOutput, timeStart, timeEnd} = minifyWithTerser({
      filePath,
      option,
      logger
    });
    const sizeDelta = sizeOutput - sizeSource;
    totalSizeSource += sizeSource;
    totalSizeDelta += sizeDelta;
    _main.__VERBOSE__ && table.push([ `∆ ${(100 * sizeDelta / sizeSource).toFixed(2)}% (${(0, _format.binary)(sizeDelta)}B)`, (0, _format.time)(timeEnd - timeStart), `${(0, _path.relative)(rootPath, filePath)}` ]);
  }
  _main.__VERBOSE__ && table.push([ "--", "--", "--" ]);
  table.push([ `∆ ${(100 * totalSizeDelta / totalSizeSource).toFixed(2)}% (${(0, _format.binary)(totalSizeDelta)}B)`, (0, _format.time)((0, _time.clock)() - totalTimeStart), `TOTAL of ${fileList.length} file (${(0, _format.binary)(totalSizeSource)}B)` ]);
  logger.log(`result:\n  ${(0, _format.padTable)({
    table,
    padFuncList: [ "L", "R", "L" ],
    cellPad: " | ",
    rowPad: "\n  "
  })}`);
  return totalSizeDelta;
});