var _path = require("path");
var _fs = require("fs");
var _webpack = (obj = require("webpack")) && obj.__esModule ? obj : {
  default: obj
};
var obj;
var _format = require("dr-js/library/common/format");
var _File = require("dr-js/library/node/file/File");
var _ExitListener = require("dr-js/library/node/system/ExitListener");
var _main = require("./main");
const getStatsCheck = (onError, onStats) => (error, statsData) => {
  if (error) return onError(error);
  if (statsData.hasErrors() || statsData.hasWarnings()) {
    const {errors = [], warnings = []} = statsData.toJson();
    errors.forEach(message => console.error(message));
    warnings.forEach(message => console.warn(message));
    if (statsData.hasErrors()) return onError(new Error("webpack stats Error"));
  }
  onStats(statsData);
};
const joinTag = (...args) => args.filter(Boolean).join(",");
exports.compileWithWebpack = (async ({config, isWatch, profileOutput, assetMapOutput, logger}) => {
  const {log} = logger;
  if (profileOutput) {
    isWatch && console.warn("[watch] warning: skipped generate profileOutput");
    config.profile = !0;
  }
  const compiler = (0, _webpack.default)(config);
  const logStats = ((isWatch, {padLog, log}) => {
    const logSingleStats = ({compilation: {assets = {}, chunks = []}, startTime, endTime}) => {
      startTime && endTime && padLog(`[${isWatch ? "watch" : "compile"}] time: ${(0, _format.time)(endTime - startTime)}`);
      const table = [];
      Object.entries(assets).forEach(([name, sourceInfo]) => table.push([ "asset", name, `${(0, _format.binary)(sourceInfo.size())}B`, joinTag(sourceInfo.emitted && "emitted") ]));
      _main.__VERBOSE__ && chunks.forEach(chunk => table.push([ "chunk", chunk.name || chunk.id, `${(0, _format.binary)(chunk.modulesSize())}B`, joinTag(chunk.canBeInitial() && "initial", chunk.hasRuntime() && "entry", chunk.rendered && "rendered") ]));
      log(`output:\n  ${(0, _format.padTable)({
        table,
        padFuncList: [ "L", "R", "R", "L" ],
        cellPad: " | ",
        rowPad: "\n  "
      })}`);
    };
    return statsData => {
      if (statsData.compilation) return logSingleStats(statsData);
      if (statsData.stats) return statsData.stats.map(logSingleStats);
      console.warn("[getLogStats] unexpected statData", statsData);
      throw new Error("[getLogStats] unexpected statData");
    };
  })(isWatch, logger);
  if (!isWatch) {
    log("[compile] start");
    const statsData = await new Promise((resolve, reject) => compiler.run(getStatsCheck(reject, resolve)));
    logStats(statsData);
    let statsDataObject;
    const getStatsDataObject = () => {
      void 0 === statsDataObject && (statsDataObject = statsData.toJson());
      return statsDataObject;
    };
    profileOutput && (0, _fs.writeFileSync)(profileOutput, JSON.stringify(getStatsDataObject()));
    profileOutput && log(`[compile] generated profileOutput at: ${profileOutput}`);
    assetMapOutput && (0, _fs.writeFileSync)(assetMapOutput, JSON.stringify(getStatsDataObject().assetsByChunkName || {}));
    assetMapOutput && log(`[compile] generated assetMapOutput at: ${assetMapOutput}`);
    return statsData;
  }
  log("[watch] start");
  compiler.watch({
    aggregateTimeout: 512,
    poll: void 0
  }, getStatsCheck(error => log(`error: ${error}`), logStats));
  (0, _ExitListener.addExitListenerSync)(exitState => log(`[watch] exit with state: ${JSON.stringify(exitState)}`));
});
exports.commonFlag = (async ({argvFlag, fromRoot, profileOutput = fromRoot(".temp-gitignore/profile-stat.json"), assetMapOutput = "", logger: {log}}) => {
  const mode = argvFlag("development", "production") || "production";
  const isWatch = Boolean(argvFlag("watch"));
  const isProduction = "production" === mode;
  argvFlag("profile") || (profileOutput = null);
  profileOutput && await (0, _File.createDirectory)((0, _path.dirname)(profileOutput));
  assetMapOutput && await (0, _File.createDirectory)((0, _path.dirname)(assetMapOutput));
  log(`compile flag: ${JSON.stringify({
    mode,
    isWatch,
    isProduction,
    profileOutput,
    assetMapOutput
  }, null, "  ")}`);
  return {
    mode,
    isWatch,
    isProduction,
    profileOutput,
    assetMapOutput
  };
});